from netbox.api.viewsets import NetBoxModelViewSet
from netbox.api.metadata import ContentTypeMetadata

from ..models import SopInfra
from ..filtersets import SopInfraFilterset
from .serializers import SopInfraSerializer


__all__ = (
    'SopInfraViewSet',
)


class SopInfraViewSet(NetBoxModelViewSet):
    metadata_class = ContentTypeMetadata
    queryset = SopInfra.objects.all()
    serializer_class = SopInfraSerializer
    filterset_class = SopInfraFilterset




from django.urls import path

from netbox.views.generic import ObjectChangeLogView, ObjectJournalView

from .views import *
from .models import *


app_name = 'sop_infra'


urlpatterns = [

    path('<int:pk>/', SopInfraDetailView.as_view(), name='sopinfra_detail'),
    path('add/', SopInfraAddView.as_view(), name='sopinfra_add'),
    path('add/<int:pk>/', SopInfraAddView.as_view(), name='sopinfra_add'),
    path('edit/<int:pk>/', SopInfraEditView.as_view(), name='sopinfra_edit'),
    path('delete/<int:pk>/', SopInfraDeleteView.as_view(), name='sopinfra_delete'),
    path('refresh/', SopInfraRefreshView.as_view(), name='sopinfra_refresh'),
    path('refresh_site/', SopInfraRefreshNoForm.as_view(), name='sopinfra_refresh_site'),
    path('journal/<int:pk>', ObjectJournalView.as_view(), name='sopinfra_journal', kwargs={'model': SopInfra}),
    path('changelog/<int:pk>', ObjectChangeLogView.as_view(), name='sopinfra_changelog', kwargs={'model': SopInfra}),

    #____________________
    # classification edit
    path('class/add/', SopInfraClassificationAddView.as_view(), name='class_add'),
    path('class/add/<int:pk>', SopInfraClassificationAddView.as_view(), name='class_add'),
    path('class/edit/<int:pk>', SopInfraClassificationEditView.as_view(), name='class_edit'),

    #____________________
    # sizing edit
    path('sizing/add/', SopInfraSizingAddView.as_view(), name='sizing_add'),
    path('sizing/add/<int:pk>', SopInfraSizingAddView.as_view(), name='sizing_add'),
    path('sizing/edit/<int:pk>', SopInfraSizingEditView.as_view(), name='sizing_edit'),

    #____________________
    # meraki sdwan edit
    path('meraki/add/', SopInfraMerakiAddView.as_view(), name='meraki_add'),
    path('meraki/add/<int:pk>', SopInfraMerakiAddView.as_view(), name='meraki_add'),
    path('meraki/edit/<int:pk>', SopInfraMerakiEditView.as_view(), name='meraki_edit'),

    #____________________
    # list views
    path('list/', SopInfraListView.as_view(), name='sopinfra_list'),
    path('class/list/', SopInfraClassificationListView.as_view(), name='class_list'),
    path('sizing/list/', SopInfraSizingListView.as_view(), name='sizing_list'),
    path('meraki/list/', SopInfraMerakiListView.as_view(), name='meraki_list'),

    #____________________
    # bulk views
    path('delete/', SopInfraBulkDeleteView.as_view(), name='sopinfra_bulk_delete'),
    path('edit/', SopInfraBulkEditView.as_view(), name='sopinfra_bulk_edit')

]




from django.db import models
from django.urls import reverse
from django.utils.safestring import mark_safe
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from netbox.models import NetBoxModel
from utilities.choices import ChoiceSet
from dcim.models import Site, Location

from .validators.model_validators import (
    DC_status_site_fields,
    SopInfraSlaveValidator,
    SopInfraMasterValidator
)


__all__ = (
    'SopInfra',
    'InfraBoolChoices',
    'InfraTypeChoices',
    'InfraTypeIndusChoices',
    'InfraHubOrderChoices',
    'InfraSdwanhaChoices'
)


class InfraBoolChoices(ChoiceSet):

    CHOICES = (
        ('unknown', _('Unknown'), 'gray'),
        ('true', _('True'), 'green'),
        ('false', _('False'), 'red'),
    )


class InfraTypeChoices(ChoiceSet):

    CHOICES = (
        ('box', _('Simple BOX server')),
        ('superb', _('Super Box')),
        ('sysclust', _('Full cluster')),
    )


class InfraTypeIndusChoices(ChoiceSet):

    CHOICES = (
        ('wrk', _('WRK - Workshop')),
        ('fac', _('FAC - Factory')),
    )


class InfraHubOrderChoices(ChoiceSet):

    CHOICES = (
        ('N_731271989494311779,L_3689011044769857831,N_731271989494316918,N_731271989494316919', 'EQX-NET-COX-DDC'),
        ('N_731271989494316918,N_731271989494316919,N_731271989494311779,L_3689011044769857831', 'COX-DDC-EQX-NET'),
        ('L_3689011044769857831,N_731271989494311779,N_731271989494316918,N_731271989494316919', 'NET-EQX-COX-DDC'),
        ('N_731271989494316919,N_731271989494316918,N_731271989494311779,L_3689011044769857831', 'DDC-COX-EQX-NET'),
    )


class InfraSdwanhaChoices(ChoiceSet):

    CHOICES = (
        ('-HA-', _('-HA-')),
        ('-NHA-', _('-NHA-')),
        ('-NO NETWORK-', _('-NO NETWORK-')),
        ('-SLAVE SITE-', _('-SLAVE SITE-')),
        ('-DC-', _('-DC-')),
    )


class SopInfra(NetBoxModel):
    site = models.OneToOneField(
        to=Site,
        on_delete=models.CASCADE,
        unique=True,
        verbose_name=_('Site')
    )
    # ______________
    # Classification
    site_infra_sysinfra = models.CharField(
        choices=InfraTypeChoices,
        null=True,
        blank=True,
        verbose_name=_('System infrastructure'),
    )
    site_type_indus = models.CharField(
        choices=InfraTypeIndusChoices,
        null=True,
        blank=True,
        verbose_name=_('Industrial'),
    )
    criticity_stars = models.CharField(
        max_length=6,
        null=True,
        blank=True,
        verbose_name=_('Criticity stars')
    )
    site_phone_critical = models.CharField(
        choices=InfraBoolChoices,
        null=True,
        blank=True,
        verbose_name=_('PHONE Critical ?'),
        help_text=_('Is the phone critical for this site ?')
    )
    site_type_red = models.CharField(
        choices=InfraBoolChoices,
        null=True,
        blank=True,
        verbose_name=_('R&D ?'),
        help_text=_('Does the site have and R&D department or a lab ?')
    )
    site_type_vip = models.CharField(
        choices=InfraBoolChoices,
        null=True,
        blank=True,
        verbose_name=_('VIP ?'),
        help_text=_('Does the site host VIPs ?')
    )
    site_type_wms = models.CharField(
        choices=InfraBoolChoices,
        null=True,
        blank=True,
        verbose_name=_('WMS ?'),
        help_text=_('Does the site run WMS ?')
    )
    #_______
    # Sizing
    est_cumulative_users = models.PositiveBigIntegerField(
        null=True,
        blank=True,
        verbose_name=_('Est. cumul. users')
    )
    site_user_count = models.CharField(
        null=True,
        blank=True,
        help_text=_('Site user count')
    )
    wan_reco_bw = models.PositiveBigIntegerField(
        null=True,
        blank=True,
        verbose_name=_('Reco. BW (Mbps)'),
        help_text=_('Recommended bandwidth (Mbps)')       
    )
    site_mx_model = models.CharField(
        max_length=6,
        null=True,
        blank=True,
        verbose_name=_('Reco. MX Model'),
    )
    wan_computed_users = models.PositiveBigIntegerField(
        null=True,
        blank=True,
        verbose_name=_('WAN users'),
        help_text=_('Total computed wan users.')
    )
    ad_direct_users = models.PositiveBigIntegerField(
        null=True,
        blank=True,
        verbose_name=_('AD direct. users')
    )
    #_______
    # Meraki
    sdwanha = models.CharField(
        choices=InfraSdwanhaChoices,
        null=True,
        blank=True,
        verbose_name=_('HA(S) / NHA target'),
        help_text=_('Calculated target for this site'),
    )
    hub_order_setting = models.CharField(
        choices=InfraHubOrderChoices,
        null=True,
        blank=True,
        verbose_name=_('HUB order setting'),
        help_text=_('Choose one of the various supported combinations')
    )
    hub_default_route_setting = models.CharField(
        choices=InfraBoolChoices,
        null=True,
        blank=True,
        verbose_name=_('HUB default route setting'),
        help_text=_('Set to true if the default route should be sent through the AutoVPN'),
    )
    sdwan1_bw = models.CharField(
        null=True,
        blank=True,
        verbose_name=_('WAN1 BW'),
        help_text=_('SDWAN > WAN1 Bandwidth (real link bandwidth)'),
    )
    sdwan2_bw = models.CharField(
        null=True,
        blank=True,
        verbose_name=_('WAN2 BW'),
        help_text=_('SDWAN > WAN2 Bandwidth (real link bandwidth)')
    )
    site_sdwan_master_location = models.ForeignKey(
        to=Location,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_('MASTER Location'),
        help_text=_('When this site is an SDWAN SLAVE, you have to materialize a location on the MASTER site and link it here')
    )
    master_site = models.ForeignKey(
        to=Site,
        related_name="master_site",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_('MASTER Site'),
        help_text=_('Or select the MASTER site.')
    )
    migration_sdwan = models.CharField(
        null=True,
        blank=True,
        verbose_name=_('Migration date'),
        help_text=_('SDWAN > Site migration date to SDWAN')
    )
    monitor_in_starting = models.CharField(
        choices=InfraBoolChoices,
        null=True,
        blank=True,
        verbose_name=_('Monitor in starting'),
        help_text=_('Centreon > Start monitoring when starting the site')
    )

    def __str__(self):
        return f'{self.site} Infrastructure'

    def get_absolute_url(self) -> str:
        return reverse('plugins:sop_infra:sopinfra_detail', args=[self.pk])

    # get_object_color methods are used by NetBoxTable
    # to display choices colors
    def get_site_type_red_color(self) -> str:
        return InfraBoolChoices.colors.get(self.site_type_red)

    def get_site_type_vip_color(self) -> str:
        return InfraBoolChoices.colors.get(self.site_type_vip)

    def get_site_type_wms_color(self) -> str:
        return InfraBoolChoices.colors.get(self.site_type_wms)

    def get_site_phone_critical_color(self) -> str:
        return InfraBoolChoices.colors.get(self.site_phone_critical)

    def get_hub_default_route_setting_color(self) -> str:
        return InfraBoolChoices.colors.get(self.hub_default_route_setting)

    def get_monitor_in_starting_color(self) -> str:
        return InfraBoolChoices.colors.get(self.hub_default_route_setting)

    def get_criticity_stars(self) -> str | None:

        if self.criticity_stars is None:
            return None

        html: str = [
            '<span class="mdi mdi-star-outline" style="color: rgba(218, 165, 32, 1);"></span>' 
            for _ in self.criticity_stars
        ]
        return mark_safe(''.join(html))

    class Meta(NetBoxModel.Meta):
        verbose_name = _('Infrastructure')
        verbose_name_plural = _('Infrastructures')
        constraints = [
            models.UniqueConstraint(
                fields=['site'],
                name='%(app_label)s_%(class)s_unique_site',
                violation_error_message=_('This site has already an Infrastrcture.')
            ),
            # PostgreSQL doesnt provide database-level constraints with related fields
            # That is why i cannot check if site == master_location__site on db level, only with clean()
            models.CheckConstraint(
                check=~models.Q(site=models.F('master_site')),
                name='%(app_label)s_%(class)s_master_site_equal_site',
                violation_error_message=_('SDWAN MASTER site cannot be itself')
            )
        ]

    def compute_wan_cumulative_users(self, instance) -> int:

        base:int = instance.wan_computed_users

        # check if this is a master site
        targets = SopInfra.objects.filter(master_site=instance.site)

        if targets.exists():
            # if it is, ad slave's wan computed user to master site
            for target in targets:
                base += target.wan_computed_users

        return base

    def clean(self):
        '''
        plenty of validators and auto-compute methods in this clean()

        to keep the code readable, cleaning methods are
        separated in class in validators.py file.
        '''

        super().clean()

        # just to be sure, should never happens
        if self.site is None:
            raise ValidationError({
                'site': 'Infrastructure must be set on a site.'
            })

        # dc site__status related validators
        if self.site.status == 'dc':
            DC_status_site_fields(self)
            return

        # all slave related validators
        SopInfraSlaveValidator(self)

        # all non-slave related validators
        SopInfraMasterValidator(self)




import django_filters
from django.db.models import Q
from django.utils.translation import gettext_lazy as _

from dcim.models import Site, Region, SiteGroup, Location
from dcim.choices import SiteStatusChoices
from netbox.filtersets import NetBoxModelFilterSet
from utilities.filters import TreeNodeMultipleChoiceFilter

from .models import SopInfra


__all__ = (
    'SopInfraFilterset',
)


class SopInfraFilterset(NetBoxModelFilterSet):
    site_id = django_filters.ModelMultipleChoiceFilter(
        queryset=Site.objects.all(),
        field_name='site'
    )
    site_name = django_filters.ModelMultipleChoiceFilter(
        queryset=Site.objects.all(),
        field_name='site__name',
    )
    status = django_filters.MultipleChoiceFilter(
        choices=SiteStatusChoices,
        field_name='site__status',
    )
    region_id = TreeNodeMultipleChoiceFilter(
        queryset=Region.objects.all(),
        field_name='site__region',
        lookup_expr='in'
    )
    group_id = TreeNodeMultipleChoiceFilter(
        queryset=SiteGroup.objects.all(),
        field_name='site__group',
        lookup_expr='in'
    )

    class Meta:
        model = SopInfra
        fields = ('id', 'site', 'site_id', 'site_name', 'status', 
                  'site_infra_sysinfra', 'site_type_indus',
                  'criticity_stars', 'site_phone_critical',
                  'site_type_red', 'site_type_vip', 'site_type_wms', 
                  'est_cumulative_users', 'site_user_count', 'wan_reco_bw',
                  'site_mx_model', 'wan_computed_users', 'ad_direct_users',
                  'sdwanha', 'hub_order_setting', 'hub_default_route_setting',
                  'sdwan1_bw', 'sdwan2_bw', 'site_sdwan_master_location',
                  'master_site', 'migration_sdwan')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(site__name__icontains=value) |
            Q(site__status__icontains=value) |
            Q(site_infra_sysinfra__icontains=value) |
            Q(site_type_indus__icontains=value) |
            Q(criticity_stars__icontains=value) |
            Q(est_cumulative_users__icontains=value) |
            Q(site_user_count__icontains=value) |
            Q(wan_reco_bw__icontains=value) |
            Q(site_mx_model__icontains=value) |
            Q(wan_computed_users__icontains=value) |
            Q(ad_direct_users__icontains=value) |
            Q(sdwanha__icontains=value) |
            Q(hub_order_setting__icontains=value) |
            Q(hub_default_route_setting__icontains=value) |
            Q(sdwan1_bw__icontains=value) |
            Q(sdwan2_bw__icontains=value) |
            Q(site_sdwan_master_location__name__icontains=value) |
            Q(master_site__name__icontains=value) |
            Q(migration_sdwan__icontains=value)
        )




from django.utils.translation import gettext_lazy as _

from netbox.registry import registry
from netbox.navigation import *
from netbox.navigation.menu import MENUS


INFRA = Menu(
    label=_('Infrastructure'),
    icon_class="mdi mdi-router-network-wireless",
    groups=(
        MenuGroup(
            label=_('Infrastructure'),
            items=(
                MenuItem(
                    link=f'plugins:sop_infra:class_list',
                    link_text=_('Classifications'),
                    permissions=[f'sop_infra.view_sopinfra'],
                    buttons=(
                        MenuItemButton(
                            link=f'plugins:sop_infra:class_add',
                            title='Add',
                            icon_class='mdi mdi-plus-thick',
                            permissions=[f'sop_infra.add_sopinfra'],
                        ),
                    ),
                ),
                MenuItem(
                    link=f'plugins:sop_infra:sizing_list',
                    link_text=_('Sizings'),
                    permissions=[f'sop_infra.view_sopinfra'],
                    buttons=(
                        MenuItemButton(
                            link=f'plugins:sop_infra:sopinfra_refresh',
                            title='Recompute',
                            icon_class='mdi mdi-refresh',
                            permissions=[f'sop_infra.change_sop_infra'],
                        ),
                        MenuItemButton(
                            link=f'plugins:sop_infra:sizing_add',
                            title='Add',
                            icon_class='mdi mdi-plus-thick',
                            permissions=[f'sop_infra.add_sopinfra'],
                        ),
                    ),
                ),
                MenuItem(
                    link=f'plugins:sop_infra:meraki_list',
                    link_text=_('Meraki SDWAN'),
                    permissions=[f'sop_infra.view_sopinfra'],
                    buttons=(
                        MenuItemButton(
                            link=f'plugins:sop_infra:meraki_add',
                            title='Add',
                            icon_class='mdi mdi-plus-thick',
                            permissions=[f'sop_infra.add_sopinfra']
                        ),
                    ),
                ),
            ),
        ),
    ),
)


MENUS.append(INFRA)




# Generated by Django 5.0.9 on 2024-10-23 08:27

import django.db.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dcim', '0190_nested_modules'),
        ('extras', '0121_customfield_related_object_filter'),
    ]

    operations = [
        migrations.CreateModel(
            name='SopInfra',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('site_infra_sysinfra', models.CharField(blank=True, null=True)),
                ('site_type_industrial', models.CharField(blank=True, null=True)),
                ('site_phone_critical', models.CharField(blank=True, null=True)),
                ('site_type_red', models.CharField(blank=True, null=True)),
                ('site_type_vip', models.CharField(blank=True, null=True)),
                ('site_type_wms', models.CharField(blank=True, null=True)),
                ('ad_cumul_user', models.PositiveBigIntegerField(blank=True, null=True)),
                ('est_cumulative_users', models.PositiveBigIntegerField(blank=True, null=True)),
                ('wan_reco_bw', models.PositiveBigIntegerField(blank=True, null=True)),
                ('wan_computed_users', models.PositiveBigIntegerField(blank=True, null=True)),
                ('swanha', models.CharField(blank=True, null=True)),
                ('hub_order_setting', models.CharField(blank=True, null=True)),
                ('hub_default_route_setting', models.CharField(blank=True, null=True)),
                ('sdwan1_bw', models.CharField(blank=True, null=True)),
                ('sdwan2_bw', models.CharField(blank=True, null=True)),
                ('migration_sdwan', models.CharField(blank=True, null=True)),
                ('monitor_in_starting', models.CharField(blank=True, null=True)),
                ('site', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='dcim.site')),
                ('site_sdwan_master_location', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='dcim.location')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'Infrastructure',
                'verbose_name_plural': 'Infrastructures',
                'abstract': False,
            },
        ),
        migrations.AddConstraint(
            model_name='sopinfra',
            constraint=models.UniqueConstraint(fields=('site',), name='sop_infra_sopinfra_unique_site', violation_error_message='This site has already an Infrastrcture.'),
        ),
    ]



import logging

from django.db.models.signals import post_save
from django.dispatch import receiver
from django.contrib import messages
from django.conf import settings

from netbox.plugins import PluginTemplateExtension
from netbox.context import current_request
from dcim.models import Site

from sop_infra.models import SopInfra



#_________SITE_POST_SAVE


@receiver(post_save, sender=Site)
def create_or_update_sopinfra(sender, instance, created, **kwargs):
    '''
    when creating or updating a Site
    create or update its related SopInfra instance
    '''
    request = current_request.get()
    target = SopInfra.objects.filter(site=instance)

    # create
    if created and not target.exists():
        infra = SopInfra.objects.create(site=instance)
        infra.full_clean()
        infra.snapshot()
        infra.save()
        try:
            messages.success(request, f'Created {infra}')
        except:pass
        return

    # update
    infra = target.first()
    infra.snapshot()
    infra.full_clean()
    infra.save()
    try:
        messages.success(request, f'Updated {infra}')
    except:pass



#_________PANELS_CONFIGURATION


ALLOWED_PANELS = ['meraki', 'classification', 'sizing']
ALLOWED_POSITIONS = ['left_page', 'right_page']


def create_new_panel(self):

    def get_extra_context() -> dict:

        qs = SopInfra.objects.filter(site=self.context['object'])
        infra = qs.first() if qs.exists() else SopInfra

        return {'infra': infra, 'what': self.what}

    return self.render(f'sop_infra/panels/panel.html', get_extra_context())


class SopInfraDashboard:

    template_name = 'sop_infra/tab/{}.html'
    # model to display dashboard on
    model = 'dcim.site'

    def __init__(self):
        self.settings = settings.PLUGINS_CONFIG.get('sop_infra', {})
        self.extensions = self.get_display_extensions()


    def get_html_panel(self, panel):

        return self.template_name.format(panel)


    def get_display_position(self, panel, display):

        if exists := display.get(panel):

            if exists not in ALLOWED_POSITIONS:
                return None

            return exists

        return None


    def get_display_extensions(self):

        extensions = []
        _display = self.settings.get('display')

        # no configuration
        if _display is None:
            return

        # error handling
        if not isinstance(_display, dict):
            logging.error(f'Invalid syntax "{_display}", must be a dict.')
            return

        for panel in _display:

            if panel not in ALLOWED_PANELS:
                logging.error(f'Invalid panel "{panel}", valid display are:', ALLOWED_PANELS)
                continue

            # return the position of {panel:position}
            position = self.get_display_position(panel, _display)

            if position is None:
                logging.error(f'Invalid position "{position}", valid positions are:', ALLOWED_POSITIONS)
                continue

            # creates dynamically a template extension class
            new_class = type(
                f'{panel}_SopInfra_panel_extension',
                (PluginTemplateExtension,), {
                    'model': self.model,
                    'what': self.get_html_panel(panel),
                    position: create_new_panel
                }
            )
            extensions.append(new_class)


        return extensions


    def push(self):
        return self.extensions



template_extensions = SopInfraDashboard().push()




# Generated by Django 5.0.9 on 2024-11-05 12:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sop_infra', '0008_alter_sopinfra_site_mx_model'),
    ]

    operations = [
        migrations.AddField(
            model_name='sopinfra',
            name='ad_direct_users',
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
    ]



from django.contrib import messages

from sop_infra.validators.model_validators import SopInfraSizingValidator
from sop_infra.models import SopInfra


__all__ = (
    'SopInfraRefreshMixin',
    'SopInfraRelatedModelsMixin'
)


class SopInfraRefreshMixin:

    def update_master_instance(self, instance, wan):

        if instance.wan_computed_users == wan:
            return

        instance.snapshot()
        instance.full_clean()
        instance.save()


    def update_child_instance(self, instance, wan):

        if instance.wan_computed_users == wan:
            return

        instance.snapshot()
        instance.full_clean()
        instance.save()


    def pre_compute_queryset(self, queryset, parent=False):

        sizing = SopInfraSizingValidator()

        for instance in queryset:

            if parent is False:
                self.update_child_instance(
                    instance,
                    sizing.get_wan_computed_users(instance)
                )
                continue

            wan = sizing.get_wan_computed_users(instance)
            instance.wan_computed_users = wan if wan is not None else 0
            self.update_master_instance(
                instance,
                instance.compute_wan_cumulative_users(instance)
            )


    def refresh_infra(self, request, queryset):

        if queryset.first() is None:
            messages.error(request, 'Please select at least one site to refresh.')
            return

        # cannot select DC because 
        queryset = queryset.exclude(site__status='dc')
        if queryset.first() is None:
            messages.error(request, 'You cannot recompute sizing on -DC- status sites.')
            return

        slave = queryset.filter(master_site__isnull=False)
        maybe_master = queryset.filter(master_site__isnull=True)

        self.pre_compute_queryset(slave, False)
        self.pre_compute_queryset(maybe_master, True)

        messages.success(request, f"Successfully updated {queryset.count()} infrastructures.")


class SopInfraRelatedModelsMixin:


    def normalize_queryset(self, obj):

        qs = [str(item) for item in obj]
        if qs == []:
            return None

        return f'id=' + '&id='.join(qs)


    def get_slave_sites(self, infra):
        '''
        look for slaves sites and join their id
        '''
        if not infra.exists():
            return None, None

        # get every SopInfra instances with master_site = current site
        # and prefetch the only attribute that matters to optimize the request
        sites = SopInfra.objects.filter(master_site=(infra.first()).site).prefetch_related('site')
        count = sites.count()

        target = sites.values_list('site__pk', flat=True)
        if not target:
            return None, None
        
        return self.normalize_queryset(target), count


    def get_slave_infra(self, infra):

        if not infra.exists():
            return None

        infras = SopInfra.objects.filter(master_site=(infra.first().site))
        count = infras.count()

        target = infras.values_list('id', flat=True)
        if not target:
            return None, None

        return self.normalize_queryset(target), count




from django import forms
from django.db.models import Q
from django.urls import reverse
from django.contrib import messages
from django.utils.translation import gettext_lazy as _

from utilities.forms.fields import DynamicModelChoiceField
from netbox.forms import NetBoxModelFilterSetForm, NetBoxModelForm
from utilities.forms.widgets import DatePicker
from utilities.forms.rendering import FieldSet
from netbox.context import current_request
from utilities.forms import add_blank_choice
from dcim.models import Site, Location, Region, SiteGroup
from dcim.choices import SiteStatusChoices

from .models import *


__all__ = (
    'SopInfraForm',
    'SopInfraMerakiForm',
    'SopInfraMerakiFilterForm',
    'SopInfraSizingForm',
    'SopInfraFilterForm',
    'SopInfraSizingFilterForm',
    'SopInfraClassificationForm',
    'SopInfraClassificationFilterForm',
    'SopInfraRefreshForm',
)


class SopInfraClassificationForm(NetBoxModelForm):

    site = DynamicModelChoiceField(
        label=_('Site'),
        queryset=Site.objects.all(),
        required=True
    )
    site_infra_sysinfra = forms.ChoiceField(
        label=_('Infrastructure'),
        choices=add_blank_choice(InfraTypeChoices),
        required=False
    )
    site_type_indus = forms.ChoiceField(
        label=_('Industrial'),
        choices=add_blank_choice(InfraTypeIndusChoices),
        required=False
    )
    site_phone_critical = forms.ChoiceField(
        label=_('Phone critical'),
        choices=add_blank_choice(InfraBoolChoices),
        required=False,
        help_text=_('Is the phone critical for this site ?')
    )
    site_type_red = forms.ChoiceField(
        label=_('R&D'),
        choices=add_blank_choice(InfraBoolChoices),
        required=False,
        help_text=_('Does the site have and R&D department or a lab ?')
    )
    site_type_vip = forms.ChoiceField(
        label=_('VIP'),
        choices=add_blank_choice(InfraBoolChoices),
        required=False,
        help_text=_('Does the site host VIPs ?')
    )
    site_type_wms = forms.ChoiceField(
        label=_('WMS'),
        choices=add_blank_choice(InfraBoolChoices),
        required=False,
        help_text=_('Does the site run WMS ?')
    )

    class Meta:
        model = SopInfra
        fields = ['site', 'site_infra_sysinfra', 'site_type_indus',
                'site_phone_critical', 'site_type_red', 'site_type_vip', 'site_type_wms']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if 'tags' in self.fields:
            del self.fields['tags']


class SopInfraMerakiForm(NetBoxModelForm):

    site = DynamicModelChoiceField(
        label=_('Site'),
        queryset=Site.objects.all(),
        required=True
    )
    sdwanha = forms.ChoiceField(
        label=_('HA(S) / NHA target'),
        help_text=_('Calculated target for this site'),
        widget=forms.Select(
            attrs={
'disabled':'disabled'
            }
        ),
        required=False
    )
    hub_order_setting = forms.ChoiceField(
        label=_('HUB order setting'),
        choices=add_blank_choice(InfraHubOrderChoices),
        initial='',
        help_text=_('Choose one of the various supported combinations'),
        required=False
    )
    hub_default_route_setting = forms.ChoiceField(
        label=_('HUB default route setting'),
        choices=add_blank_choice(InfraBoolChoices),
        initial='',
        help_text=_('Set to true if the default route should be sent through the AutoVPN'),
        required=False
    )
    sdwan1_bw = forms.CharField(
        label=_('WAN1 BW'),
        help_text=_('SDWAN > WAN1 Bandwidth (real link bandwidth)'),
        required=False
    )
    sdwan2_bw = forms.CharField(
        label=_('WAN2 BW'),
        help_text=_('SDWAN > WAN2 Bandwidth (real link bandwidth)'),
        required=False
    )
    site_sdwan_master_location = DynamicModelChoiceField(
        label=_('MASTER Location'),
        queryset=Location.objects.all(),
        help_text=_('When this site is an SDWAN SLAVE, you have to materialize a location on the MASTER site and link it here'),
        required=False
    )
    master_site = forms.CharField(
        label=_('MASTER Site'),
        help_text=_('Or select the MASTER site.'),
        required=False
    )
    migration_sdwan = forms.DateField(
        label=_('Migration SDWAN'),
        widget=DatePicker(),
        help_text=_('SDWAN > Site migration date to SDWAN'),
        required=False
    )
    monitor_in_starting = forms.ChoiceField(
        label=_('Monitor in starting'),
        choices=add_blank_choice(InfraBoolChoices),
        help_text=_('Centreon > Start monitoring when starting the site'),
        required=False
    )

    class Meta:
        model = SopInfra
        fields = ['site', 'sdwanha', 'hub_order_setting', 'hub_default_route_setting',
                  'sdwan1_bw', 'sdwan2_bw', 'site_sdwan_master_location', 'master_site',
                  'migration_sdwan', 'monitor_in_starting']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if 'tags' in self.fields:
            del self.fields['tags']


class SopInfraSizingForm(NetBoxModelForm):

    site = DynamicModelChoiceField(
        label=_('Site'),
        queryset=Site.objects.all(),
        required=True
    )
    est_cumulative_users = forms.IntegerField(
        label=_('EST cumul. users'),
        required=False
    )

    class Meta:
        model = SopInfra
        fields = ['site', 'est_cumulative_users']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if 'tags' in self.fields:
            del self.fields['tags']


class SopInfraForm(
    SopInfraClassificationForm,
    SopInfraMerakiForm,
    SopInfraSizingForm
):

    site = DynamicModelChoiceField(
        label=_('Site'),
        queryset=Site.objects.all(),
        required=True
    )
    fieldsets = (
        FieldSet(
            'site', name=_('Site')
        ),
        FieldSet(
            'site_infra_sysinfra', 'site_type_indus', 'site_phone_critical',
            'site_type_red', 'site_type_vip', 'site_type_wms',
            name=_('Classification')
        ),
        FieldSet(
            'est_cumulative_users',
            name=_('Sizing')
        ),
        FieldSet(
            'sdwanha', 'hub_order_setting', 'hub_default_route_setting',
            'sdwan1_bw', 'sdwan2_bw', 'site_sdwan_master_location',
            'master_site', 'migration_sdwan', 'monitor_in_starting',
            name=_('Meraki SDWAN')
        )
    )

    class Meta:
        model = SopInfra
        fields = [
            'site', 'site_infra_sysinfra', 'site_type_indus', 'site_phone_critical',
            'site_type_red', 'site_type_vip', 'site_type_wms',
            'est_cumulative_users',
            'sdwanha', 'hub_order_setting', 'hub_default_route_setting',
            'sdwan1_bw', 'sdwan2_bw', 'site_sdwan_master_location',
            'master_site', 'migration_sdwan', 'monitor_in_starting',
        ]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if 'tags' in self.fields:
            del self.fields['tags']


#_____________
# filter forms


#_____________
# template to avoid code-repetition
class SopInfraBaseFilterForm(NetBoxModelFilterSetForm):
    model = SopInfra

    site_id = DynamicModelChoiceField(
        queryset=Site.objects.all(),
        required=False,
        label=_('Site')
    )
    region_id = DynamicModelChoiceField(
        queryset=Region.objects.all(),
        required=False,
        label=_('Region')
    )
    group_id = DynamicModelChoiceField(
        queryset=SiteGroup.objects.all(),
        required=False,
        label=_('Site group')
    )
    status = forms.ChoiceField(
        choices=add_blank_choice(SiteStatusChoices),
        initial=None,
        required=False,
        label=_('Status')
    )


class SopInfraMerakiFilterForm(SopInfraBaseFilterForm):
    sdwanha = forms.ChoiceField(
        label=_('HA(S) / NHA target'),
        choices=add_blank_choice(InfraSdwanhaChoices),
        required=False
    )
    hub_order_setting = forms.ChoiceField(
        label=_('HUB order setting'),
        choices=add_blank_choice(InfraHubOrderChoices),
        initial='',
        required=False
    )
    hub_default_route_setting = forms.ChoiceField(
        label=_('HUB default route setting'),
        choices=add_blank_choice(InfraBoolChoices),
        initial='',
        required=False
    )
    sdwan1_bw = forms.CharField(
        label=_('WAN1 BW'),
        required=False
    )
    sdwan2_bw = forms.CharField(
        label=_('WAN2 BW'),
        required=False
    )
    site_sdwan_master_location = DynamicModelChoiceField(
        label=_('MASTER Location'),
        queryset=Location.objects.all(),
        required=False
    )
    master_site = DynamicModelChoiceField(
        label=_('MASTER Site'),
        queryset=Site.objects.all(),
        required=False
    )
    migration_sdwan = forms.DateField(
        label=_('Migration SDWAN'),
        widget=DatePicker(),
        required=False
    )
    monitor_in_starting = forms.ChoiceField(
        label=_('Monitor in starting'),
        choices=add_blank_choice(InfraBoolChoices),
        required=False
    )

    fieldsets = (
        FieldSet(
            'region_id', 'group_id', 'site_id',
            name=_('Location')
        ),
        FieldSet(
            'status',
            name=_('Status')
        ),
        FieldSet(
            'sdwanha', 'hub_order_setting', 'hub_default_route_setting', 'sdwan1_bw',
            'sdwan2_bw', 'site_sdwan_master_location', 'master_site', 'migration_sdwan',
            'monitor_in_starting',
            name=_('Attributes')
        )
    )



class SopInfraClassificationFilterForm(SopInfraBaseFilterForm):
    site_infra_sysinfra = forms.ChoiceField(
        label=_('Infrastructure'),
        choices=add_blank_choice(InfraTypeChoices),
        required=False
    )
    site_type_indus = forms.ChoiceField(
        label=_('Industrial'),
        choices=add_blank_choice(InfraTypeIndusChoices),
        required=False
    )
    site_phone_critical = forms.ChoiceField(
        label=_('Phone critical'),
        choices=add_blank_choice(InfraBoolChoices),
        required=False,
    )
    site_type_red = forms.ChoiceField(
        label=_('R&D'),
        choices=add_blank_choice(InfraBoolChoices),
        required=False,
    )
    site_type_vip = forms.ChoiceField(
        label=_('VIP'),
        choices=add_blank_choice(InfraBoolChoices),
        required=False,
    )
    site_type_wms = forms.ChoiceField(
        label=_('WMS'),
        choices=add_blank_choice(InfraBoolChoices),
        required=False,
    )

    fieldsets = (
        FieldSet(
            'region_id', 'group_id', 'site_id',
            name=_('Location')
        ),
        FieldSet(
            'status',
            name=_('Status')
        ),
        FieldSet(
            'site_infra_sysinfra', 'site_type_indus', 'site_phone_critical',
            'site_type_red', 'site_type_vip', 'site_type_wms',
            name=_('Attributes')
        )
    )


class SopInfraSizingFilterForm(SopInfraBaseFilterForm):
    ad_direct_users = forms.IntegerField(
        required=False,
        label=_('AD direct. users'),
        help_text=_('Numbers only')
    )
    est_cumulative_users = forms.IntegerField(
        required=False,
        label=_('EST cumul. users'),
        help_text=_('Numbers only')
    )
    site_user_count = forms.CharField(
        required=False,
        label=_('Site user count'),
        help_text=_('Example: 50<100')
    )
    site_mx_model = forms.CharField(
        required=False,
        label=_('Reco. MX Model'),
        help_text=_('Example: MX85')
    )
    wan_reco_bw = forms.IntegerField(
        required=False,
        label=_('Reco. BW (Mbps)'),
        help_text=_('Numbers only')
    )
    wan_computed_users = forms.IntegerField(
        required=False,
        label=_('WAN computed users'),
        help_text=_('Numbers only')
    )

    fieldsets = (
        FieldSet(
            'region_id', 'group_id', 'site_id',
            name=_('Location')
        ),
        FieldSet(
            'status',
            name=_('Status')
        ),
        FieldSet(
            'ad_direct_users', 'est_cumulative_users',
            'wan_computed_users', 'wan_reco_bw',
            'site_user_count', 'site_mx_model',
            name=_('Attributes')
        )
    )


class SopInfraFilterForm(
    SopInfraClassificationFilterForm,
    SopInfraMerakiFilterForm,
    SopInfraSizingFilterForm):

    fieldsets = (
        FieldSet(
            'site', name=_('Site')
        ),
        FieldSet(
            'site_infra_sysinfra', 'site_type_indus', 'site_phone_critical',
            'site_type_red', 'site_type_vip', 'site_type_wms',
            name=_('Classification')
        ),
        FieldSet(
            'ad_direct_users', 'est_cumulative_users',
            'wan_computed_users', 'wan_reco_bw',
            'site_user_count', 'site_mx_model',
            name=_('Sizing')
        ),
        FieldSet(
            'sdwanha', 'hub_order_setting', 'hub_default_route_setting',
            'sdwan1_bw', 'sdwan2_bw', 'site_sdwan_master_location',
            'master_site', 'migration_sdwan', 'monitor_in_starting',
            name=_('Meraki SDWAN')
        )
    )


class SopInfraRefreshForm(forms.Form):

    site = DynamicModelChoiceField(
        queryset=Site.objects.all(),
        required=False
    )
    region = DynamicModelChoiceField(
        queryset=Region.objects.all(),
        required=False
    )
    group = DynamicModelChoiceField(
        queryset=SiteGroup.objects.all(),
        required=False
    )
  
    def clean(self):
        data = super().clean()
        sites = Site.objects.none()
        base_url = reverse('plugins:sop_infra:sizing_list')
        request = current_request.get()


        def get_group_sites(obj):

            ids = (obj.get_descendants()).values_list('sites', flat=True)
            sites = Site.objects.filter(group=obj.pk)
            if ids.exists():
                sites = Site.objects.filter(id__in=ids)

            if not sites.exists():
                messages.error(request, f'No sites has been found on {(obj._meta.verbose_name).title()} : {obj}')
                raise forms.ValidationError({'site': f"No sites has been found on {obj}"})

            return sites


        def get_region_sites(obj):

            ids = (obj.get_descendants()).values_list('sites', flat=True)
            sites = Site.objects.filter(region=obj.pk)
            if ids.exists():
                sites = Site.objects.filter(id__in=ids)

            if not sites.exists():
                messages.error(request, f'No sites has been found on {(obj._meta.verbose_name).title()} : {obj}')
                raise forms.ValidationError({'site': f"No sites has been found on {obj}"})

            return sites


        def normalize_queryset(obj):

            qs = [str(item) for item in obj]
            if qs == []:
                return None

            return f'id=' + '&id='.join(qs)


        if data['region']:
            sites |= get_region_sites(data['region'])

        if data['group']:
            sites |= get_group_sites(data['group'])

        if sites.filter(status='dc').exists():
            messages.warning(
                request, f"{' '.join(str(site.name) for site in sites.filter(status='dc'))} \
 skipped: You cannot recompute sizing on -DC- status site."
            )

        infra = SopInfra.objects.filter(site__in=(sites.exclude(status='dc').distinct()))

        return {
            'infra': infra,
            'return_url': f"{base_url}?{normalize_queryset(infra.values_list('id', flat=True))}"
        }




# Generated by Django 5.0.9 on 2024-11-18 13:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sop_infra', '0009_sopinfra_ad_direct_users'),
    ]

    operations = [
        migrations.RenameField(
            model_name='sopinfra',
            old_name='ad_cumul_user',
            new_name='ad_cumulative_users',
        ),
    ]



from netbox.plugins import PluginConfig


class SopInfraConfig(PluginConfig):
    name = "sop_infra"
    verbose_name = "SOP Infra"
    description = "Manage infrastructure informations of each site"
    version = "0.3.2"
    author = "Leorevoir"
    author_email = "leo.quinzler@epitech.eu"
    base_url = "sop-infra"
    min_version = '4.1.0'


config = SopInfraConfig




from django.urls import reverse

from utilities.testing import TestCase
from dcim.models import Site

from sop_infra.models import SopInfra


__all__ = (
    'SopInfraViewTestCase',
)


VIEW_PERM = 'sop_infra.view_sopinfra'
ADD_PERM = 'sop_infra.add_sopinfra'
EDIT_PERM = 'sop_infra.change_sopinfra'


class SopInfraViewTestCase(TestCase):

    user_permissions = ()

    @classmethod
    def setUpTestData(cls):

        sites = (
            Site(name='site 1', slug='site-1', status='active'),
            Site(name='site 2', slug='site-2', status='retired'),
        )
        for site in sites:
            site.full_clean()
            site.save()


    def get_action_url(self, action, instance=None):
        """reverse sopinfra plugin url with action"""
        url = f'plugins:sop_infra:sopinfra_{action}'

        if instance is None:
            return reverse(url)

        return reverse(url, kwargs={'pk': instance.pk})


    def test_list_no_perm(self):
        """test list view no perm"""
        url = self.get_action_url('list')

        response = self.client.get(url)
        self.assertHttpStatus(response, 403)


    def test_list_perm(self):
        """test list view perm"""
        url = self.get_action_url('list')

        self.add_permissions(VIEW_PERM)
        response = self.client.get(url)
        self.assertHttpStatus(response, 200)


    def test_add_no_perm(self):
        """test add view no perm"""
        url = self.get_action_url('add')

        response = self.client.get(url)
        self.assertHttpStatus(response, 403)


    def test_add_perm(self):
        """test add view perm"""
        url = self.get_action_url('add')

        self.add_permissions(ADD_PERM)
        response = self.client.get(url)
        self.assertHttpStatus(response, 200)


    def test_detail_no_perm(self):
        """test detail no perm"""
        instance = SopInfra.objects.first()
        url = self.get_action_url('detail', instance)

        response = self.client.get(url)
        self.assertHttpStatus(response, 403)


    def test_detail_perm(self):
        """test detail perm"""
        instance = SopInfra.objects.first()
        url = self.get_action_url('detail', instance)

        self.add_permissions(VIEW_PERM)
        response = self.client.get(url)
        self.assertHttpStatus(response, 200)


    def test_edit_no_perm(self):
        """test edit no perm"""
        instance = SopInfra.objects.first()
        url = self.get_action_url('edit', instance)

        response = self.client.get(url)
        self.assertHttpStatus(response, 403)


    def test_edit_perm(self):
        """test detail perm"""
        instance = SopInfra.objects.first()
        url = self.get_action_url('edit', instance)

        self.add_permissions(EDIT_PERM)
        response = self.client.get(url)
        self.assertHttpStatus(response, 200)


    def test_tab_view_no_perm(self):
        """test tab no perm"""
        instance = Site.objects.first()
        url = f'/dcim/sites/{instance.pk}/infra/'

        self.add_permissions('dcim.view_site')
        response = self.client.get(url)
        self.assertHttpStatus(response, 403)


    def test_tab_view_perm(self):
        """test tab perm"""
        instance = Site.objects.first()
        url = f'/dcim/sites/{instance.pk}/infra/'

        self.add_permissions('dcim.view_site')
        self.add_permissions(VIEW_PERM)
        response = self.client.get(url)
        self.assertHttpStatus(response, 200)


    def test_tab_view_context(self):
        """test tab view context"""
        site = Site.objects.first()
        infra = SopInfra.objects.get(site=site)
        url = f'/dcim/sites/{site.pk}/infra/'
        s_ids = {}
        i_ids = {}

        for i in range(3):
            s = Site.objects.create(name=f'salut{i}', slug=f'salut{i}')
            s.full_clean()
            s.save()
            s_ids[i] = s.pk
            t = SopInfra.objects.get(site__id=s_ids[i])
            t.master_site=site
            t.full_clean()
            t.save()
            i_ids[i] = t.pk

        self.add_permissions('dcim.view_site')
        self.add_permissions(VIEW_PERM)
        response = self.client.get(url)
        context = response.context['context']

        self.assertHttpStatus(response, 200)
        self.assertEqual(context['sop_infra'], infra)
        self.assertEqual(context['count_slave_infra'], 3)
        self.assertEqual(context['count_slave'], 3)
        self.assertEqual(context['slave'], f'id={s_ids[0]}&id={s_ids[1]}&id={s_ids[2]}')
        self.assertEqual(context['slave_infra'], f'id={i_ids[0]}&id={i_ids[1]}&id={i_ids[2]}')


    def test_recompute_sizing_no_perm(self):
        """test recompute sizing no perm"""
        instance = Site.objects.first()
        url = self.get_action_url('refresh')

        response = self.client.get(url)
        self.assertHttpStatus(response, 403)

        response = self.client.get(f'{url}?qs={instance.pk}')
        self.assertHttpStatus(response, 403)




# Generated by Django 5.0.9 on 2024-11-20 09:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sop_infra', '0010_rename_ad_cumul_user_sopinfra_ad_cumulative_users'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='sopinfra',
            name='ad_cumulative_users',
        ),
    ]



from django.db.models import Prefetch
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from netbox.api.fields import ChoiceField
from netbox.api.serializers import NetBoxModelSerializer
from dcim.api.serializers import SiteSerializer, LocationSerializer
from dcim.models import Site, Location

from ..models import *


__all__ = (
    'SopInfraSerializer',
)



class SopInfraSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:sop_infra-api:sopinfra-detail')
    site = serializers.SerializerMethodField()
    sdwanha = ChoiceField(choices=InfraSdwanhaChoices)
    hub_order_setting = ChoiceField(InfraHubOrderChoices)
    site_sdwan_master_location = serializers.SerializerMethodField()
    master_site = serializers.SerializerMethodField()
    site_infra_sysinfra = ChoiceField(choices=InfraTypeChoices)
    site_type_indus = ChoiceField(choices=InfraTypeIndusChoices)
    
    class Meta:
        model = SopInfra
        fields = (
            'id', 'url', 'display', 'site',
            'site_infra_sysinfra', 'site_type_indus', 'site_phone_critical',
            'site_type_red', 'site_type_vip', 'site_type_wms', 'criticity_stars',
            'ad_direct_users', 
            'est_cumulative_users', 'wan_reco_bw', 'wan_computed_users',
            'site_mx_model',
            'sdwanha', 'hub_order_setting', 'hub_default_route_setting',
            'sdwan1_bw', 'sdwan2_bw', 'site_sdwan_master_location',
            'master_site', 'migration_sdwan', 'monitor_in_starting',
            'created', 'last_updated',
        )
        brief_fields = ('id', 'url', 'display', 'site_infra_sysinfra', 'criticity_stars',
                        'site_type_indus', 'sdwanha', 'site_sdwan_master_location', 'master_site')

    def get_site(self, obj):
        if not obj.site:
            return None
        return SiteSerializer(obj.site, nested=True, many=False, context=self.context).data

    def get_site_sdwan_master_location(self, obj):
        if not obj.site_sdwan_master_location:
            return None
        return LocationSerializer(obj.site_sdwan_master_location, nested=True, many=False, context=self.context).data

    def get_master_site(self, obj):
        if not obj.master_site:
            return None
        return SiteSerializer(obj.master_site, nested=True, many=False, context=self.context).data




# Generated by Django 5.0.9 on 2024-10-29 10:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0190_nested_modules'),
        ('extras', '0121_customfield_related_object_filter'),
        ('sop_infra', '0005_sopinfra_site_user_count'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='sopinfra',
            constraint=models.CheckConstraint(check=models.Q(('site', models.F('master_site')), _negated=True), name='sop_infra_sopinfra_master_site_equal_site', violation_error_message='SDWAN MASTER site cannot be itself'),
        ),
    ]



from django.db import models
import django_tables2 as tables
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from netbox.tables import NetBoxTable, ChoiceFieldColumn
from dcim.models import Site
from dcim.choices import SiteStatusChoices

from .models import SopInfra


__all__ = (
    'SopInfraTable',
    'SopInfraSizingTable',
    'SopInfraMerakiTable',
    'SopInfraClassificationTable'
)


class SopInfraMerakiTable(NetBoxTable):
    '''
    table for all SopInfra - meraki sdwan related instances
    '''
    site = tables.Column(linkify=True)
    status = tables.Column(accessor='site__status', linkify=True)
    sdwanha = tables.Column(linkify=True)
    hub_order_setting = tables.Column(linkify=True)
    hub_default_route_setting = ChoiceFieldColumn(linkify=True)
    sdwan1_bw = tables.Column(linkify=True)
    sdwan2_bw = tables.Column(linkify=True)
    site_sdwan_master_location = tables.Column(linkify=True)
    master_site = tables.Column(linkify=True)
    migration_sdwan = tables.Column(linkify=True)
    monitor_in_starting = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = SopInfra
        fields = (
            'actions', 'pk', 'id', 'created', 'last_updated', 'site', 'status',
            'sdwanha', 'hub_order_setting', 'hub_default_route_setting',
            'sdwan1_bw', 'sdwan2_bw', 'site_sdwan_master_location',
            'master_site', 'migration_sdwan', 'monitor_in_starting'
        )
        default_columns = (
            'actions', 'site', 'status', 'sdwanha', 'hub_order_setting',
            'hub_default_route_setting', 'sdwan1_bw', 'sdwan2_bw'
        )
        order_by = ('site',)

    def render_status(self, record):
        if not record.site:
            return None
        
        value = record.site.status
        bg_color = SiteStatusChoices.colors.get(value)
        if not bg_color:
            bg_color = "gray"
        return mark_safe(f'<span class="badge text-bg-{bg_color}">{value.title()}</span>')


class SopInfraSizingTable(NetBoxTable):
    '''
    table for all SopInfra - sizing related instances
    '''
    site = tables.Column(linkify=True)
    status = tables.Column(accessor='site__status', linkify=True)
    est_cumulative_users = tables.Column(linkify=True)
    wan_computed_users = tables.Column(linkify=True)
    wan_reco_bw = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = SopInfra
        fields = (
            'actions', 'pk', 'id', 'created', 'last_updated', 'site', 'status',
            'est_cumulative_users', 'wan_computed_users', 'wan_reco_bw',
            'ad_direct_users', 'site_mx_model'
        )
        default_columns = (
            'site', 'status', 'est_cumulative_users',
            'wan_computed_users', 'wan_reco_bw', 'site_mx_model'
        )
        order_by = ('site',)

    def render_status(self, record):
        if not record.site:
            return None
        value = record.site.status
        bg_color = SiteStatusChoices.colors.get(value)
        if not bg_color:
            bg_color = "gray"
        return mark_safe(f'<span class="badge text-bg-{bg_color}">{value.title()}</span>')


class SopInfraClassificationTable(NetBoxTable):
    '''
    table for all SopInfra - classification related instances
    '''
    site = tables.Column(linkify=True)
    status = tables.Column(accessor='site__status', linkify=True)
    site_infra_sysinfra = tables.Column(linkify=True)
    site_type_indus = tables.Column(linkify=True)
    criticity_stars = tables.Column(linkify=True)
    site_phone_critical = ChoiceFieldColumn(linkify=True)
    site_type_red = ChoiceFieldColumn(linkify=True)
    site_type_vip = ChoiceFieldColumn(linkify=True)
    site_type_wms = ChoiceFieldColumn(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = SopInfra
        fields = (
            'actions', 'pk', 'id', 'created', 'last_updated', 'site', 'status'
            'site_infra_sysinfra', 'site_type_indus', 'site_phone_critical',
            'site_type_red', 'site_type_vip', 'site_type_wms',
        )
        default_columns = (
            'site', 'status', 'site_infra_sysinfra', 'site_type_indus',
            'site_phone_critical',
            'site_type_red', 'site_type_vip', 'site_type_wms', 'criticity_stars',
        )
        order_by = ('site',)

    def render_status(self, record):
        if not record.site:
            return None
        
        value = record.site.status
        bg_color = SiteStatusChoices.colors.get(value)
        if not bg_color:
            bg_color = "gray"
        return mark_safe(f'<span class="badge text-bg-{bg_color}">{value.title()}</span>')

    def render_criticity_stars(self, record):
        if not record.criticity_stars:
            return None

        return record.get_criticity_stars()


class SopInfraTable(
    SopInfraClassificationTable,
    SopInfraMerakiTable,
    SopInfraSizingTable):

    class Meta(NetBoxTable.Meta):
        model = SopInfra
        fields = (
            'actions', 'pk', 'id', 'created', 'last_updated', 'site', 'status'
            'site_infra_sysinfra', 'site_type_indus', 'site_phone_critical',
            'site_type_red', 'site_type_vip', 'site_type_wms',
            'est_cumulative_users', 'wan_computed_users', 'wan_reco_bw',
            'ad_direct_users', 'site_mx_model',
            'sdwanha', 'hub_order_setting', 'hub_default_route_setting',
            'sdwan1_bw', 'sdwan2_bw', 'site_sdwan_master_location',
            'master_site', 'migration_sdwan', 'monitor_in_starting'
        )
        default_columns = (
            'site', 'status',
            'sdwanha', 'ad_direct_users', 'wan_computed_users'
        )

        order_by = ('site',)




# Generated by Django 5.0.9 on 2024-11-05 09:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sop_infra', '0007_sopinfra_criticity_stars_sopinfra_site_mx_model'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sopinfra',
            name='site_mx_model',
            field=models.CharField(blank=True, max_length=6, null=True),
        ),
    ]



from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError


__all__ = (
    'SopInfraMasterValidator',
    'SopInfraSlaveValidator',
    'DC_status_site_fields',
)


def DC_status_site_fields(instance) -> None:

    instance.sdwanha = '-DC-'
    instance.site_user_count = '-DC'
    instance.site_sdwan_master_location = None
    instance.master_site = None
    instance.wan_reco_bw = None
    instance.wan_computed_users = None
    instance.criticity_stars = '****'
    instance.site_mx_model = 'MX450'



class SopInfraSizingValidator:
    '''
    SopInfra - sizing related auto-computes validator methods
    '''

    @staticmethod
    def get_wan_computed_users(instance) -> int:

        if instance.site.status in ['active', 'decommissioning']:
            return instance.ad_direct_users

        elif instance.site.status in ['candidate', 'planned', 'staging']:
            return instance.est_cumulative_users

        elif instance.site.status in ['starting']:
            wan = instance.ad_direct_users

            if wan is None:
                wan = 0

            if instance.est_cumulative_users is not None and instance.est_cumulative_users > wan:
                return instance.est_cumulative_users

            return instance.ad_direct_users

        return 0


    @staticmethod
    def get_mx_and_user_slice(wan:int) -> tuple[str]:

        if wan < 10 :
            return '<10', 'MX67'
        elif wan < 20 :
            return '10<20', 'MX67'
        elif wan < 50 :
            return '20<50', 'MX68'
        elif wan < 100 :
            return '50<100', 'MX85'
        elif wan < 200 :
            return '100<200', 'MX95'
        elif wan < 500 :
            return '200<500', 'MX95'
        return '>500', 'MX250'


    @staticmethod
    def get_recommended_bandwidth(wan:int) -> int:

        if wan > 100:
            return round(wan * 2.5)
        elif wan > 50:
            return round(wan * 3)
        elif wan > 10:
            return round(wan * 4)
        else:
            return round(wan * 5)



class SopInfraSlaveValidator:

    def __init__(self, instance):

        if instance.site_sdwan_master_location and instance.master_site is None:
            instance.master_site = instance.site_sdwan_master_location.site

        # if not slave -> return
        if instance.master_site is None:
            return

        self.check_master_exists(instance)
        self.check_no_loop(instance)
        self.compute_slave_sizing(instance)
        self.reset_slave_fields(instance)


    def check_master_exists(self, instance):

        from sop_infra.models import SopInfra
        target = SopInfra.objects.exclude(pk=instance.pk)

        if instance.site_sdwan_master_location is not None:
            if target.filter(site_sdwan_master_location=instance.site_sdwan_master_location).exists():
                raise ValidationError({
                    'site_sdwan_master_location': 'This location is already the MASTER location for other sites infrastructures.'
                })


    def check_no_loop(self, instance):

        if instance.site_sdwan_master_location is not None:
            if instance.site_sdwan_master_location.site == instance.site:
                raise ValidationError({
                    'site_sdwan_master_location': 'SDWAN MASTER site cannot be itself'
                 })

            if instance.site_sdwan_master_location.site != instance.master_site:
                 raise ValidationError({
                    'site_sdwan_master_location': 'SDWAN MASTER location site must be equal to MASTER site or leaved blank.'
                 })

        if instance.master_site == instance.site:
            raise ValidationError({
                'master_site': 'SDWAN MASTER site cannot be itself'
            })


    def compute_slave_sizing(self, instance):

        sizing = SopInfraSizingValidator()

        wan = sizing.get_wan_computed_users(instance)
        if wan is None:
            wan = 0
        instance.wan_computed_users = wan


    def reset_slave_fields(self, instance):

        instance.sdwanha = '-SLAVE SITE-'
        instance.sdwan1_bw = None
        instance.sdwan2_bw = None
        instance.migration_sdwan = None
        instance.site_type_vip = None
        instance.site_type_wms = None
        instance.site_type_red = None
        instance.site_phone_critical = None
        instance.site_infra_sysinfra = None
        instance.site_type_indus = None
        instance.wan_reco_bw = None
        instance.criticity_stars = None
        instance.site_mx_model = None



class SopInfraMasterValidator:

    def __init__(self, instance):

        if self.is_slave(instance):
            return

        self.compute_master_sizing(instance)
        self.compute_sdwanha(instance)


    def is_slave(self, instance) -> bool:

        return instance.master_site is not None \
            or instance.site_sdwan_master_location is not None


    def compute_master_sizing(self, instance):

        sizing = SopInfraSizingValidator()

        # base wan
        wan = sizing.get_wan_computed_users(instance)
        if wan is None:
            wan = 0
        instance.wan_computed_users = wan

        # additional wan
        wan = instance.compute_wan_cumulative_users(instance)
        instance.wan_computed_users = wan

        # user slice
        user_slice, mx = sizing.get_mx_and_user_slice(wan)
        instance.site_user_count = user_slice
        instance.site_mx_model = mx

        # reco bw
        bw = sizing.get_recommended_bandwidth(wan)
        instance.wan_reco_bw = bw


    def compute_sdwanha(self, instance):

        if instance.site.status in [
            'no_infra', 'candidate', 'reserved',
            'template', 'inventory', 'teleworker']:
        # enforce no_infra constraints
            instance.sdwanha = '-NO NETWORK-'
            instance.sdwan1_bw = None
            instance.sdwan2_bw = None
            instance.criticity_stars = None
            instance.site_infra_sysinfra = None
        else:
            # compute sdwanha for normal sites
            instance.sdwanha = '-NHA-'
            instance.criticity_stars = '*'
            if instance.site_type_vip == 'true':
                instance.sdwanha = '-HA-'
                instance.criticity_stars = '***'
            # no -HAS- because there is no site_type_indus == IPL
            elif instance.site_type_indus == 'fac' \
                or instance.site_phone_critical == 'true' \
                or instance.site_type_red == 'true' \
                or instance.site_type_wms == 'true' \
                or instance.site_infra_sysinfra == 'sysclust' \
                or instance.site_user_count in ['50<100', '100<200', '200<500', '>500']:
                instance.sdwanha = '-HA-'
                instance.criticity_stars = '**'




from django import template


__all__ = (
    'field_verbose_name',
    'field_help_text',
)


register = template.Library()


@register.filter()
def field_verbose_name(instance, field_name):
    '''
    html filter to return field verbose_name
    usage: object|field_verbose_name:"field_name"
    '''
    try:
        return instance._meta.get_field(field_name).verbose_name
    except:
        return None


@register.filter()
def field_help_text(instance, field_name):
    '''
    html filter to return field help_text if exists
    usage: object|field_help_text:"field_name"
    '''
    try:
        return instance._meta.get_field(field_name).help_text
    except:
        return None


@register.filter()
def field_value(instance, field_name):
    '''
    html filter to return field value
    usage: object|field_value:"field_name"
    '''
    try:
        maybe_method = f'get_{field_name}_display'
        if hasattr(instance, maybe_method):
            return getattr(instance, maybe_method)()

        maybe_method = f'get_{field_name}'
        if hasattr(instance, maybe_method):
            return getattr(instance, maybe_method)()

        return getattr(instance, field_name)
    except:
        return None


@register.filter()
def field_exists(instance, field_name) -> bool:
    '''
    html filter to see if object exists
    usage: object|field_exists:"field_name"
    '''
    try:
        if hasattr(instance, field_name):
            if getattr(instance, field_name) is not None:
                return True

        return False
    except:
        return False


@register.filter()
def field_help_exists(instance, field_name) -> bool:
    '''
    html filter to see if object has help_text
    usage: object|field_help_exists:"field_name"
    '''
    try:
        field = instance._meta.get_field(field_name)
        text = field.help_text
        if text:
            return True
        return False
    except:
        return False




from setuptools import setup, find_packages


setup(
    name='sop-infra',
    version='0.3.2',
    packages=find_packages(),
    include_package_data=True,
    description = "Manage infrastructure informations of each site.",
    author="Leorevoir",
    author_email="leo.quinzler@epitech.eu",
    classifiers=[
        'Framework :: Django',
        'Programming Language :: Python :: 3',
    ],
    zip_safe=False,
)



from django.core.exceptions import ValidationError
from django.db import IntegrityError, transaction

from utilities.testing import TestCase
from dcim.models import Site, Location

from sop_infra.models import SopInfra


__all__ = (
    'SopInfraSlaveModelTestCase',
    'SopInfraMasterModelTestCase'
)


class SopInfraSlaveModelTestCase(TestCase):

    user_permissions = ()


    @classmethod
    def setUpTestData(cls):

        sites = (
            Site(name='site 1', slug='site-1', status='active'),
            Site(name='site 2', slug='site-2', status='starting'),
            Site(name='site 3', slug='site-3', status='staging')
        )
        for site in sites:
            site.save()

        location = Location(site=Site.objects.first(), name="test-location", slug='test-loc')
        location.full_clean()
        location.save()

        cls.site1 = Site.objects.get(slug='site-1')
        cls.site2 = Site.objects.get(slug='site-2')
        cls.site3 = Site.objects.get(slug='site-3')
        cls.location = location


    def test_slave_wrong_location(self):
        """Test that invalid master location raises ValidationError"""
        with self.assertRaises(ValidationError):
            infra = SopInfra.objects.get(site=self.site1)
            infra.site_sdwan_master_location = self.location
            infra.full_clean()
            infra.save()


    def test_slave_wrong_master_site(self):
        """Test that invalid master site raises IntegrityError"""
        with transaction.atomic():
            with self.assertRaises(IntegrityError):
                infra = SopInfra.objects.get(site=self.site1)
                infra.master_site=self.site1
                infra.save()


    def test_slave_master_location_site_not_master_site(self):
        """Test master site coherence """
        with self.assertRaises(ValidationError):
            infra = SopInfra.objects.get(site=self.site1)
            infra.site_sdwan_master_location=self.location
            infra.master_site=self.site3
            infra.full_clean()
            infra.save()


    def test_slave_correct_master_location(self):
        """Test valid master location"""
        infra = SopInfra.objects.get(site=self.site2)
        infra.site_sdwan_master_location=self.location
        infra.full_clean()
        self.assertEqual(infra.master_site, infra.site_sdwan_master_location.site)


    def test_slave_correct_master_site(self):
        """Test valid master site"""
        infra = SopInfra.objects.get(site=self.site2)
        infra.master_site=self.site1
        infra.full_clean()
        self.assertEqual(infra.site_sdwan_master_location, None)


    def test_slave_master_location_already_exists(self):
        """Test that if master location already exists -> raise ValidationError"""
        with self.assertRaises(ValidationError):
            infra = SopInfra.objects.get(site=self.site2)
            infra.site_sdwan_master_location=self.location
            infra.full_clean()
            infra.save()

            infra2 = SopInfra.objects.get(site=self.site3)
            infra2.site_sdwan_master_location=self.location
            infra2.full_clean()


    def test_slave_compute_sizing(self):
        """Test that valid slave infra computes sizing"""
        infra = SopInfra.objects.get(site=self.site1)
        infra.master_site=self.site3

        infra.full_clean()
        self.assertEqual(infra.wan_computed_users, 0)

        infra.ad_direct_users = 42
        infra.est_cumulative_users = 69
        infra.full_clean()
        self.assertEqual(infra.wan_computed_users, 42)

        infra = SopInfra.objects.get(site=self.site2)
        infra.ad_direct_users = 42
        infra.est_cumulative_users = 69

        infra.full_clean()
        self.assertEqual(infra.wan_computed_users, 69)


    def test_slave_default_fields(self):
        """Test that valid slave infra computes default fields"""
        infra = SopInfra.objects.get(site=self.site1)
        infra.master_site=self.site2
        infra.full_clean()
        self.assertEqual(infra.sdwanha, '-SLAVE SITE-')

        infra.master_site = None
        infra.full_clean()
        self.assertEqual(infra.sdwanha, '-NHA-')



class SopInfraMasterModelTestCase(TestCase):

    user_permissions = ()


    @classmethod
    def setUpTestData(cls):

        sites = (
            Site(name='site 1', slug='site-1', status='active'),
            Site(name='site 2', slug='site-2', status='starting'),
            Site(name='site 3', slug='site-3', status='staging')
        )
        for site in sites:
            site.save()

        cls.site1 = Site.objects.get(slug='site-1')
        cls.site2 = Site.objects.get(slug='site-2')
        cls.site3 = Site.objects.get(slug='site-3')

        cls.infra1 = SopInfra.objects.get(site__slug='site-1')
        cls.infra2 = SopInfra.objects.get(site__slug='site-2')
        cls.infra3 = SopInfra.objects.get(site__slug='site-3')

        cls.infra1.ad_direct_users = 442
        cls.infra2.ad_direct_users = 42
        cls.infra2.est_cumulative_users = 69
        cls.infra2.site_type_red = 'true'
        cls.infra3.ad_direct_users = 0
        cls.infra3.est_cumulative_users = 19
        cls.infra3.site_type_vip = 'true'

        cls.infra1.full_clean()
        cls.infra1.save()
        cls.infra2.full_clean()
        cls.infra2.save()
        cls.infra3.full_clean()
        cls.infra3.save()


    def test_master_wan_computed_users(self):
        """Test that valid MASTER SopInfra computes wan_computed_users"""
        self.assertEqual(self.infra1.wan_computed_users, 442)
        self.assertEqual(self.infra2.wan_computed_users, 69)
        self.assertEqual(self.infra3.wan_computed_users, 19)


    def test_master_wan_computed_users_children(self):
        """Test that valid MASTER SopInfra with children computes wan_computed_users"""
        self.infra1.master_site = self.infra2.site

        self.infra1.full_clean()
        self.infra1.save()
        self.infra2.full_clean()
        self.infra2.save()

        self.assertEqual(self.infra1.wan_computed_users, 442)
        self.assertEqual(self.infra2.wan_computed_users, 511)


    def test_master_mx_user_slice(self):
        """Test that valid MASTER SopInfra computes mx and user_slice"""

        self.assertEqual(self.infra1.site_mx_model, 'MX95')
        self.assertEqual(self.infra2.site_mx_model, 'MX85')
        self.assertEqual(self.infra3.site_mx_model, 'MX67')

        self.assertEqual(self.infra1.site_user_count, '200<500')
        self.assertEqual(self.infra2.site_user_count, '50<100')
        self.assertEqual(self.infra3.site_user_count, '10<20')


    def test_master_mx_user_slice_children(self):
        """Test that valid MASTER SopInfra with children computes mx and user_slice"""
        self.infra1.master_site = self.infra2.site
        self.infra2.ad_direct_users = 84

        self.infra1.full_clean()
        self.infra1.save()
        self.infra2.full_clean()
        self.infra1.save()

        self.assertEqual(self.infra1.site_mx_model, None)
        self.assertEqual(self.infra2.site_mx_model, 'MX250')

        self.assertEqual(self.infra1.site_user_count, '200<500')
        self.assertEqual(self.infra2.site_user_count, '>500')


    def test_master_recommended_bandwidth(self):
        """Test that valid MASTER SopInfra computes recommended bandwidth"""

        # bw = wan * cm where cm depends of wan size
        self.assertEqual(self.infra1.wan_reco_bw, round(442 * 2.5 ))
        self.assertEqual(self.infra2.wan_reco_bw, round(69 * 3))
        self.assertEqual(self.infra3.wan_reco_bw, round(19 * 4))


        self.infra2.est_cumulative_users = 9
        self.infra2.ad_direct_users = 8
        self.infra2.full_clean()
        self.infra2.save()

        self.assertEqual(self.infra2.wan_reco_bw, round(9 * 5))


    def test_master_recommended_bandwidth_children(self):
        """Test that valid MASTER SopInfra with children computes recommended bandwidth"""
        self.infra1.master_site = self.infra2.site
        self.infra2.ad_direct_users = 84

        self.infra1.full_clean()
        self.infra1.save()
        self.infra2.full_clean()
        self.infra1.save()

        self.assertEqual(self.infra1.wan_reco_bw, None)
        self.assertEqual(self.infra2.wan_reco_bw, round(526 * 2.5))


    def test_master_compute_sdwanha(self):
        """Test that valid MASTER SopInfra computes default fields"""

        self.infra2.site_type_vip = 'true'
        self.infra3.site.status = 'candidate'

        self.infra2.full_clean()
        self.infra2.save()
        self.infra3.full_clean()
        self.infra3.save()

        self.assertEqual(self.infra1.sdwanha, '-HA-')
        self.assertEqual(self.infra1.criticity_stars, '**')

        self.assertEqual(self.infra2.sdwanha, '-HA-')
        self.assertEqual(self.infra2.criticity_stars, '***')

        self.assertEqual(self.infra3.sdwanha, '-NO NETWORK-')
        self.assertEqual(self.infra3.criticity_stars, None)

        self.infra1.ad_direct_users = 9
        self.infra1.full_clean()
        self.infra1.save()

        self.assertEqual(self.infra1.sdwanha, '-NHA-')
        self.assertEqual(self.infra1.criticity_stars, '*')




from django.utils.translation import gettext_lazy as _
from django.shortcuts import render, get_object_or_404, redirect
from django.views import View
from django.urls import reverse

from utilities.views import register_model_view, ViewTab, ObjectPermissionRequiredMixin 
from utilities.permissions import get_permission_for_model
from utilities.forms import restrict_form_fields
from netbox.views import generic
from dcim.models import Site

from .forms import *
from .tables import *
from .models import *
from .filtersets import SopInfraFilterset
from .utils import SopInfraRelatedModelsMixin, SopInfraRefreshMixin


__all__ = (
    'SopInfraTabView',
    'SopInfraAddView',
    'SopInfraEditView',
    'SopInfraListView',
    'SopInfraDeleteView',
    'SopInfraDetailView',
    'SopInfraRefreshView',
    'SopInfraRefreshNoForm',
    'SopInfraBulkEditView',
    'SopInfraBulkDeleteView',
    'SopInfraMerakiAddView',
    'SopInfraMerakiEditView',
    'SopInfraMerakiListView',
    'SopInfraSizingAddView',
    'SopInfraSizingEditView',
    'SopInfraSizingListView',
    'SopInfraClassificationAddView',
    'SopInfraClassificationEditView',
    'SopInfraClassificationListView',
)


@register_model_view(Site, name='infra')
class SopInfraTabView(
    View,
    ObjectPermissionRequiredMixin,
    SopInfraRelatedModelsMixin):
    '''
    creates an "infrastructure" tab on the site page
    '''
    tab = ViewTab(label=_('Infrastructure'), permission=get_permission_for_model(SopInfra, 'view'))
    template_name: str = 'sop_infra/tab/tab.html'
    queryset = SopInfra.objects.all()

    def get_extra_context(self, request, pk) -> dict:
        context: dict = {}
        
        site = get_object_or_404(Site, pk=pk)

        infra = SopInfra.objects.filter(site=site)

        if infra.exists():
            context['sop_infra'] = infra.first()
        else:
            context['sop_infra'] = SopInfra
        context['slave'], context['count_slave'] = self.get_slave_sites(infra)
        context['slave_infra'], context['count_slave_infra'] = self.get_slave_infra(infra)
        return {'object': site, 'context': context}

    def get(self, request, pk):
        # additional permission security
        if not request.user.has_perm(get_permission_for_model(SopInfra, 'view')):
            return self.handle_no_permission()
        return render(request, self.template_name, self.get_extra_context(request, pk))


#____________________________
# SOP INFRA BASE MODEL VIEWS


class SopInfraDeleteView(generic.ObjectDeleteView):
    '''
    deletes an existing SopInfra instance
    '''
    queryset = SopInfra.objects.all()


class SopInfraEditView(generic.ObjectEditView):
    '''
    edits an existing SopInfra instance
    '''
    template_name:str = 'sop_infra/tools/forms.html'
    queryset = SopInfra.objects.all()
    form = SopInfraForm

    def get_return_url(self, request, obj):
        if obj.site:
            return f'/dcim/sites/{obj.site.id}/infra'

    def get_extra_context(self, request, obj):
        context = super().get_extra_context(request, obj)
        if not obj:
            return context
        context['object_type'] = obj
        return context


class SopInfraAddView(generic.ObjectEditView):
    '''
    adds a new SopInfra instance
    if the request is from the site page,
    -> the site id is passed as an argument (pk)
    '''
    queryset = SopInfra.objects.all()
    form = SopInfraForm

    def get_object(self, **kwargs):
        '''        
        '''
        if 'pk' in kwargs:
            site= get_object_or_404(Site, pk=kwargs['pk'])
            obj = self.queryset.model
            return obj(site=site)
        return super().get_object(**kwargs)

    def alter_object(self, obj, request, args, kwargs):
        '''
        '''
        if 'pk' in kwargs:
            site = get_object_or_404(Site, pk=kwargs['pk'])
            obj = self.queryset.model
            return obj(site=site)
        return super().alter_object(obj, request, args, kwargs)

    def get_return_url(self, request, obj):
        try:
            return f'/dcim/sites/{obj.site.id}/infra'
        except:
            return f'/plugins/sop_infra/list'


#____________________________
# CLASSIFICATION ADD/EDIT


class SopInfraClassificationAddView(generic.ObjectEditView):
    '''
    only adds classification objects in a SopInfra instance
    '''
    template_name:str = 'sop_infra/tools/forms.html'
    queryset = SopInfra.objects.all()
    form = SopInfraClassificationForm

    def get_object(self, **kwargs):
        '''        
        '''
        if 'pk' in kwargs:
            site = get_object_or_404(Site, pk=kwargs['pk'])
            obj = self.queryset.model
            return obj(site=site)
        return super().get_object(**kwargs)

    def alter_object(self, obj, request, args, kwargs):
        '''
        '''
        if 'pk' in kwargs:
            site = get_object_or_404(Site, pk=kwargs['pk'])
            obj = self.queryset.model
            return obj(site=site)
        return super().alter_object(obj, request, args, kwargs)

    def get_return_url(self, request, obj):
        try:
            if obj.site:
                return f'/dcim/sites/{obj.site.id}/infra'
        except:
            return f'/plugins/sop_infra/class/list'

    def get_extra_context(self, request, obj) -> dict:
        context = super().get_extra_context(request, obj)
        context['object_type'] = 'Classification'
        return context


class SopInfraClassificationEditView(generic.ObjectEditView):
    '''
    only edits classification objects in a sopinfra instance
    '''
    template_name:str = 'sop_infra/tools/forms.html'
    queryset = SopInfra.objects.all()
    form = SopInfraClassificationForm

    def get_return_url(self, request, obj):
        if obj.site:
            return f'/dcim/sites/{obj.site.id}/infra'

    def get_extra_context(self, request, obj) -> dict:
        context = super().get_extra_context(request, obj)
        context['object_type'] = 'Classification'
        if obj and obj.site:
            context['site'] = obj.site
        return context


#____________________________
# MERAKI SDWAN ADD/EDIT


class SopInfraMerakiAddView(generic.ObjectEditView):
    '''
    only adds meraki sdwan objects in a sopinfra instance
    '''
    template_name:str = 'sop_infra/tools/forms.html'
    queryset = SopInfra.objects.all()
    form = SopInfraMerakiForm

    def get_object(self, **kwargs):
        '''
        '''
        if 'pk' in kwargs:
            site = get_object_or_404(Site, pk=kwargs['pk'])
            obj = self.queryset.model
            return obj(site=site)
        return super().get_object(**kwargs)

    def alter_object(self, obj, request, args, kwargs):
        '''
        '''
        if 'pk' in kwargs:
            site = get_object_or_404(Site, pk=kwargs['pk'])
            obj = self.queryset.model
            return obj(site=site)
        return super().alter_object(obj, request, args, kwargs)

    def get_return_url(self, request, obj):
        try:
            if obj.site:
                return f'/dcim/sites/{obj.site.id}/infra'
        except:
            return f'/plugins/sop_infra/meraki/list'

    def get_extra_context(self, request, obj) -> dict:
        context = super().get_extra_context(request, obj)
        context['object_type'] = 'Meraki SDWAN'
        return context


class SopInfraMerakiEditView(generic.ObjectEditView):
    '''
    only edits meraki sdwan objects in a sopinfra instance
    '''
    template_name:str = 'sop_infra/tools/forms.html'
    queryset = SopInfra.objects.all()
    form = SopInfraMerakiForm
    
    def get_return_url(self, request, obj):
        if obj.site:
            return f'/dcim/sites/{obj.site.id}/infra'

    def get_extra_context(self, request, obj) -> dict:
        context = super().get_extra_context(request, obj)
        context['object_type'] = 'Meraki SDWAN'
        if obj and obj.site:
            context['site'] = obj.site
        return context


#____________________________
# SIZING ADD/EDIT


class SopInfraSizingAddView(generic.ObjectEditView):
    '''
    only adds sizing objects in a sopinfra instance
    '''
    template_name:str = 'sop_infra/tools/forms.html'
    queryset = SopInfra.objects.all()
    form = SopInfraSizingForm

    def get_object(self, **kwargs):
        '''
        '''
        if 'pk' in kwargs:
            site = get_object_or_404(Site, pk=kwargs['pk'])
            obj = self.queryset.model
            return obj(site=site)
        return super().get_object(**kwargs)

    def alter_object(self, obj, request, args, kwargs):
        '''
        '''
        if 'pk' in kwargs:
            site = get_object_or_404(Site, pk=kwargs['pk'])
            obj = self.queryset.model
            return obj(site=site)
        return super().alter_object(obj, request, args, kwargs)

    def get_return_url(self, request, obj):
        try:
            if obj.site:
                return f'/dcim/sites/{obj.site.id}/infra'
        except:
            return f'/plugins/sop_infra/sizing/list'

    def get_extra_context(self, request, obj) -> dict:
        context = super().get_extra_context(request, obj)
        context['object_type'] = 'Sizing'
        return context


class SopInfraSizingEditView(generic.ObjectEditView):
    '''
    only edits sizing objects in a sopinfra instance
    '''
    template_name:str = 'sop_infra/tools/forms.html'
    queryset = SopInfra.objects.all()
    form = SopInfraSizingForm

    def get_return_url(self, request, obj):
        if obj.site:
            return f'/dcim/sites/{obj.site.id}/infra'

    def get_extra_context(self, request, obj) -> dict:
        context = super().get_extra_context(request, obj)
        context['object_type'] = 'Sizing'
        if obj and obj.site:
            context['site'] = obj.site
        return context


#____________________________
# DETAIL VIEW


class SopInfraDetailView(generic.ObjectView):
    '''
    detail view with changelog and journal
    '''
    queryset = SopInfra.objects.all()


#____________________________
# LIST VIEWS


class SopInfraClassificationListView(generic.ObjectListView):
    '''
    list view of all sopinfra - classification related instances
    '''
    template_name:str = "sop_infra/tools/tables.html"
    queryset = SopInfra.objects.all()
    table = SopInfraClassificationTable
    filterset_form = SopInfraClassificationFilterForm
    filterset = SopInfraFilterset

    def get_extra_context(self, request) -> dict:
        '''add context for title'''
        context = super().get_extra_context(request)
        context['title'] = "Classification"
        return context


class SopInfraSizingListView(generic.ObjectListView):
    '''
    list view of all sopinfra - sizing related instances
    '''
    template_name:str = "sop_infra/tools/tables.html"
    queryset = SopInfra.objects.all()
    table = SopInfraSizingTable
    filterset = SopInfraFilterset
    filterset_form = SopInfraSizingFilterForm

    def get_extra_context(self, request) -> dict:
        '''add context for title'''
        context = super().get_extra_context(request)
        context['title'] = "Sizing"
        return context


class SopInfraMerakiListView(generic.ObjectListView):
    '''
    list view of all sopinfra - meraki sdwan related instances
    '''
    template_name:str = "sop_infra/tools/tables.html"
    queryset = SopInfra.objects.all()
    table = SopInfraMerakiTable
    filterset = SopInfraFilterset
    filterset_form = SopInfraMerakiFilterForm

    def get_extra_context(self, request) -> dict:
        '''add context for title'''
        context = super().get_extra_context(request)
        context['title'] = "Meraki SDWAN"
        return context


class SopInfraListView(generic.ObjectListView):
    '''list of all SopInfra objects and instances'''
    queryset = SopInfra.objects.all()
    table = SopInfraTable
    filterset = SopInfraFilterset
    filterset_form = SopInfraFilterForm


class SopInfraRefreshView(
    View,
    SopInfraRefreshMixin,
    ObjectPermissionRequiredMixin):
    '''
    refresh targeted sopinfra computed values
    '''
    form = SopInfraRefreshForm
    template_name:str = 'sop_infra/tools/refresh_form.html'


    def get(self, request, *args, **kwargs):
        
        # additional security
        if not request.user.has_perm(get_permission_for_model(SopInfra, 'change')):
            return self.handle_no_permission()

        restrict_form_fields(self.form(), request.user)

        return render(request, self.template_name, {
            'form': self.form(),
            'return_url': reverse('plugins:sop_infra:sizing_list')
        })

    def post(self, request, *args, **kwargs):

        # additional security
        if not request.user.has_perm(get_permission_for_model(SopInfra, 'change')):
            return self.handle_no_permission()

        return_url = reverse('plugins:sop_infra:sizing_list')
        form = self.form(data=request.POST, files=request.FILES)
        if form.is_valid():
            data:dict = form.cleaned_data
            return_url = data['return_url']
            self.refresh_infra(request, data['infra'])
            return redirect(return_url)

        return render(request, self.template_name, {
            'form': self.form(),
            'return_url': return_url
        })



class SopInfraRefreshNoForm(
    View,
    SopInfraRefreshMixin,
    ObjectPermissionRequiredMixin):

    def get_return_url(self, pk) -> str:

        return f'/dcim/sites/{pk}/infra/'

    def get(self, request, *args, **kwargs):

        # additional security
        if not request.user.has_perm(get_permission_for_model(SopInfra, 'change')):
            return self.handle_no_permission()

        pk = request.GET.get('qs')
        self.refresh_infra(request, SopInfra.objects.filter(site__pk=pk))
        return redirect(self.get_return_url(pk))

#____________________________
# bulk views


class SopInfraBulkEditView(generic.BulkEditView):

    queryset = SopInfra.objects.all()
    form = SopInfraForm
    table = SopInfraTable
    filterset = SopInfraFilterset
    filterset_form = SopInfraFilterForm


class SopInfraBulkDeleteView(generic.BulkDeleteView):

    queryset = SopInfra.objects.all()
    table = SopInfraTable
    filterset = SopInfraFilterset
    filterset_form = SopInfraFilterForm




# Generated by Django 5.0.9 on 2024-10-24 14:15

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0190_nested_modules'),
        ('sop_infra', '0003_rename_swanha_sopinfra_sdwanha'),
    ]

    operations = [
        migrations.AddField(
            model_name='sopinfra',
            name='master_site',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='master_site', to='dcim.site'),
        ),
    ]



# Generated by Django 5.0.9 on 2024-10-23 11:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sop_infra', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='sopinfra',
            old_name='site_type_industrial',
            new_name='site_type_indus',
        ),
    ]



from django.urls import reverse

from utilities.testing import TestCase
from dcim.models import Site

from sop_infra.models import SopInfra


VIEW_PERM = 'sop_infra.view_sopinfra'


class SopInfraApiTestCase(TestCase):

    user_permissions = ()

    @classmethod
    def setUpTestData(cls):

        sites = (
            Site(name='site 1', slug='site-1', status='active'),
            Site(name='site 2', slug='site-2', status='retired'),
        )
        for site in sites:
            site.full_clean()
            site.save()


    def get_action_url(self, action, instance=None):
        """reverse sopinfra plugin api url with action"""
        url = f'plugins-api:sop_infra-api:sopinfra-{action}'

        if instance is None:
            return reverse(url)

        return reverse(url, kwargs={'pk': instance.pk})


    def test_detail_no_perm(self):
        """get api object no perm"""
        instance = SopInfra.objects.first()
        url = self.get_action_url('detail', instance)

        response = self.client.get(url)
        self.assertHttpStatus(response, 403)


    def test_detail_perm(self):
        """get api object perm"""
        instance = SopInfra.objects.first()
        url = self.get_action_url('detail', instance)

        self.add_permissions(VIEW_PERM)
        response = self.client.get(url)
        self.assertHttpStatus(response, 200)


    def test_list_no_perm(self):
        """get api list no perm"""
        url = self.get_action_url('list')

        response = self.client.get(url)
        self.assertHttpStatus(response, 403)


    def test_list_perm(self):
        """get api list perm"""
        url = self.get_action_url('list')

        self.add_permissions(VIEW_PERM)
        response = self.client.get(url)
        self.assertHttpStatus(response, 200)
        self.assertEqual(len(response.data['results']), 2)

